#!/bin/bash
#-------------------------------------------------------------------------------
# This script file launches an elaboration batch on the cluster.
# It automatically creates all the files needed to launch the batch.
# Since all the needed files are created at once, scripts' maintenance should 
# be easier, faster and less prone to errors.
#-------------------------------------------------------------------------------
#                            (by Stefano Bragaglia, stefano.bragaglia@gmail.com)



# GLOBAL VARIABLES
# ----------------
# Set these variables to match target machine's configuration.

# Absolute path to YAP interpreter
if [ ! -f $YAP ]; then
	YAP="/home/collab1/bragaglia/local/bin/yap"
	# YAP="/home/rzf/usr/local/bin/yap"
fi

# If true, this flag reserves a whole cluster's machine to the batch
SINGLE=false

# Elaboration timeout
TIMEOUT=10


# ALGORITHM SPECIFIC VARIABLES
# ----------------------------
# Set these variables to achieve particular elaborations.

# Current algorithm's tagname
NAME=mc

# Current algorithm's source filename
SOURCE=montecarlo

# Current algorithm's parameters
K=64
DELTA=0.01

# Current algorithm's setup command
SETUP="set(k, $K). set(delta, $DELTA)."

# Default output filename
FILENAME="$NAME.k=$K.d=$DELTA"

# Current algorithm's calling signature
FUNCTION="solve([path(0, 1)], Samples, Time, Lower, Prob, Upper)"
MASK="~d;~3f;~6f;~6f;~6f"
VARS="Samples, Time, Lower, Prob, Upper"


# ACTUAL CODE
# -----------
# The following code should be left unchanged.
# If you want to run a particular elaboration batch, modify the above variables instead.

# Parameter's check
# -----------------
if [ $# -lt 1 ]; then 
	echo 
	echo "Syntax error:" 
	echo "	$0  <valid email address>" 
	exit
fi 
MAIL=$1

FOLDER=( branches lanes parachutes )
for ((i = 0; i < ${#FOLDER[@]}; i++)); do 

	# Creating test program file
	# --------------------------
	echo "/* -----------------------------------------------------------------------------" > ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * This prolog program runs current simulation and saves the results to a file. " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * -----------------------------------------------------------------------------" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " *                        (by Stefano Bragaglia, stefano.bragaglia@gmail.com) */" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo ":- use_module('../../../sources/$SOURCE')." >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo ":- use_module('../../../sources/parsing')." >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "/* savedata(File) " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * -------------- " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * This predicate saves the results of the simulation into the given File. " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " */ " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "savedata(File) :- " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	open(File, write, Stream), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	compute(Stream, 1), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	close(Stream). " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "/* compute(Stream, Number) " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * ----------------------- " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * This predicate writes the results for the current randomization's step of " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " * given Number to the given Stream. " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo " */ " >> ./${FOLDER[$i]}/test_$NAME.pl 
	if [ $i == 0 ]; then	
		echo "compute(_Stream, 26) :- !. " >> ./${FOLDER[$i]}/test_$NAME.pl 
	else
		echo "compute(_Stream, 301) :- !. " >> ./${FOLDER[$i]}/test_$NAME.pl 
	fi
	echo "" >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "compute(Stream, Number) :- " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	number_codes(Number, NumberCode), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	append(\"step\", NumberCode, FileName), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	atom_codes(FileNameAtom, FileName), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	parse(FileNameAtom), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	$FUNCTION, !, " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	format(Stream, \"~d;$MASK~n\", [Number, $VARS]), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	format(\"~d;$MASK~n\", [Number, $VARS]), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	flush_output(Stream), " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	Next is Number + 1, " >> ./${FOLDER[$i]}/test_$NAME.pl 
	echo "	compute(Stream, Next). " >> ./${FOLDER[$i]}/test_$NAME.pl 


	# Creating of launching command file
	# ----------------------------------
	echo "#!/bin/bash" > ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "#-------------------------------------------------------------------------------" >> ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "# This bash script launches the YAP instance for current simulation." >> ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "#-------------------------------------------------------------------------------" >> ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "#                            (by Stefano Bragaglia, stefano.bragaglia@gmail.com)" >> ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "cd `pwd`/${FOLDER[$i]}/" >> ./${FOLDER[$i]}/comm_$NAME.cmd
	echo "echo \"$SETUP savedata('times_$FILENAME.csv'). halt.\" | $YAP -l test_$NAME.pl 1> stdout_$FILENAME.txt 2> stderr_$FILENAME.txt" >> ./${FOLDER[$i]}/comm_$NAME.cmd
	chmod +x ./${FOLDER[$i]}/comm_$NAME.cmd 


	# Creating cluster config file 
	# -------------------------------
	echo "<start>" > ./${FOLDER[$i]}/conf_$NAME 
	echo "<run> 1 source `pwd`/${FOLDER[$i]}/comm_$NAME.cmd" >> ./${FOLDER[$i]}/conf_$NAME 
	if $SINGLE = true; then
		echo "<single>" >> ./${FOLDER[$i]}/conf_$NAME 
	fi
	echo "<email> $MAIL" >> ./${FOLDER[$i]}/conf_$NAME 
	echo "<timeout> $TIMEOUT" >> ./${FOLDER[$i]}/conf_$NAME 
	echo "<end>" >> ./${FOLDER[$i]}/conf_$NAME 


	# Cleaning of target directory
	# ----------------------------
	rm -f ./${FOLDER[$i]}/times_$FILENAME.csv
	rm -f ./${FOLDER[$i]}/stderr_$FILENAME.txt
	rm -f ./${FOLDER[$i]}/stdout_$FILENAME.txt

	
	# Queueing current simulation to cluster's tasklist
	# -------------------------------------------------
	cat ./${FOLDER[$i]}/conf_$NAME | gmsub
done 

# Final notification
# ------------------
echo 
echo "Simulations for $SOURCE evaluation queued!"
echo "Check $MAIL for updates..."

exit 
